#! /bin/bash
#
# Author Markus Gruenwald
# Url: www.mg-it.net
# Date 2008-09-08
#
#
#


PROGNAME="${0##*/}"
PROGPATH="${0%/*}"
VERSION="20141013"

. $PROGPATH/utils.sh


DEBUG=0
PLUGIN_TIMEOUT=10
NAGIOS_PATH="/usr/local/nagios"

PATH=$PATH:/usr/local/bin



print_usage() {
    echo ""
    echo "Usage:"
    echo ""
    echo "$PROGNAME -w <WARNING%>"
    echo ""
    echo "$PROGNAME -c <CRITICAL%>"
    echo ""
    echo "$PROGNAME --help"
    echo "$PROGNAME --version"
    echo ""
}

print_help() {
    print_head
    echo ""
    print_usage
    echo ""
    echo "What is this? Plugin for Nagios?"
    echo ""
}


print_head() {
	echo "$PROGNAME Version $VERSION"
        $ECHO "This plugin come with ABSOLUTELY NO WARRANTY. You may redistribute\ncopies of the plugins under the terms of the GNU General Public License.\n"
}




if [ $# -lt 1 ];
  then
    echo "Can not find any argument"
    print_usage
    exit $STATE_UNKNOWN
fi



while test -n "$1"; do
    case "$1" in
        --help)
            print_help
            exit $STATE_OK
            ;;
        -h)
            print_help
            exit $STATE_OK
            ;;
        --version)
            print_head
            exit $STATE_OK
            ;;
        -V)
            print_head
            exit $STATE_OK
            ;;
        -w)
            WARNING=$2
            shift
            ;;
        -c)
            CRITICAL=$2
            shift
            ;;
        *)
            echo "Unknown argument: $1"
            print_usage
            exit $STATE_UNKNOWN
            ;;
    esac
    shift
done


SAR=$(type -p sar); [[ "$(echo $?)" != "0" ]] && echo "missing sar (sysstats)" && exit 1
BC=$(type -p bc); [[ "$(echo $?)" != "0" ]] && echo "missing bc (GNU Command Line Calculator)" && exit 1

LANG=POSIX
PATH=$PATH:/usr/local/bin

DAY=$(date '+%d')
HOUR=$(date '+%H')
MINUTE=$(date '+%M')
BEFORE=$(date -d "7 minutes ago" '+%T')
AFTER=$(date -d "+1 minutes" '+%T')
INTERVAL=300 # 5 min
FILE=/var/log/sa/sa$DAY

SAR_OPT="-i $INTERVAL -s $BEFORE -e $AFTER -f $FILE"

[[ "$HOUR" == 00 ]] && (( $MINUTE <= 06 )) && SAR_OPT="0"

SAR_CMD="$SAR -u ALL $SAR_OPT"


while read LINE
  do
    if grep -q "%idle" <<< $LINE
      then
        PARAMS=(${LINE//\%/})
    elif grep -q "Average:" <<< $LINE
      then
        VALUES=(${LINE})
    fi
  done <<< "$($SAR_CMD)"


COUNTER=0
for PARAM in ${PARAMS[*]}
  do

    case "$PARAM" in
        user|usr)
            USER=${VALUES[$COUNTER]}
            ;;

        nice)
            NICE=${VALUES[$COUNTER]}
            ;;

        system|sys)
            SYSTEM=${VALUES[$COUNTER]}
            ;;

        iowait)
            IOWAIT=${VALUES[$COUNTER]}
            ;;

        idle)
            IDLE=${VALUES[$COUNTER]}
            ;;

    esac

    COUNTER=$((COUNTER+1))

  done

USED=$($BC <<< "scale=4;100 - $IDLE")


if grep "^\." <<< $USED > /dev/null
  then
    USED=$(cut -b -3 <<< $USED)
    USED="0$USED"

  else
    USED=$(cut -d "." -f 1 <<< $USED)
    USED="$USED"

fi


USED_OP=$(cut -d "." -f 1 <<< $USED)


if [[ $USED_OP -ge $CRITICAL ]]
  then
    RETVAL=$STATE_CRITICAL
    STATUS=CRITICAL

elif [[ $USED_OP -ge $WARNING ]]
  then
    RETVAL=$STATE_WARNING
    STATUS=WARNING

else
    RETVAL=$STATE_OK
    STATUS=OK

fi


RETSTR="CPU $STATUS - Used:$USED% Idle:$IDLE% (User:$USER%  Nice:$NICE%  Sys:$SYSTEM%  Wait I/O:$IOWAIT%)|used=$USED%;$WARNING;$CRITICAL;;100"

echo $RETSTR
exit $RETVAL
